@@ -141,13 +141,24 @@ class CommandLineInterface:
     def run(self, func=None):
         from .argparse import extract_command_name, parse_argv
         
+        # print(':lv', self.commands, self._cname_2_func)
+        
         config.apply_changes()
         mode: T.Mode = 'group' if not func else 'command'  # noqa
         
         if func is None:
             if cmd_name := extract_command_name(sys.argv):
                 # print(':v', cmd_name)
-                func = self._cname_2_func[cmd_name]
+                try:
+                    func = self._cname_2_func[cmd_name]
+                except KeyError:
+                    from .general import did_you_mean
+                    if x := did_you_mean(cmd_name, self._cname_2_func):
+                        console.print('[red]Command "{}" not found, did you '
+                                      'mean "{}"?[/]'.format(cmd_name, x))
+                    else:
+                        console.print(f'[red]Unknown command: {cmd_name}[/]')
+                    sys.exit(1)
         
         result = parse_argv(
             argv=sys.argv,
@@ -184,7 +195,7 @@ class CommandLineInterface:
         # FIXME: we take '--help' as the most important option to check. the
         #   '--help' is the only global option for now.
         if not result['args'] and not result['kwargs']:
-            if self.commands[id(func)]['args']:
+            if func is None or self.commands[id(func)]['args']:
                 # it means user is not providing sufficient arguments.
                 # instead of rasing an exception, we guide user to see the help
                 # message.
@@ -319,6 +330,13 @@ class CommandLineInterface:
         from rich.padding import Padding
         
         def show(func: t.Optional[t.Callable], show_logo: bool) -> dict:
+            """
+            warning:
+                this function is not the same with [self.show()]. some minor
+                changes are made.
+                if we want to merge them into one in the future, be careful
+                comparing the difference between them.
+            """
             is_group: bool
             has_args: bool
             has_kwargs: bool
@@ -384,6 +402,8 @@ class CommandLineInterface:
                 if desc:
                     from textwrap import indent
                     collect_renderables['desc'] = indent(desc, ' ')
+                elif has_args or has_kwargs:
+                    collect_renderables['desc'] = ' ' + config.FALLBACK_DESC
                 
                 if args := func_info['args']:
                     collect_renderables['arg_panel'] = (
